/* Copyright (C) 2013 TU Dortmund
 * This file is part of LearnLib, http://www.learnlib.de/.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.learnlib.logging.filter;

import java.util.EnumSet;
import java.util.logging.Filter;
import java.util.logging.LogRecord;

import de.learnlib.logging.Category;
import de.learnlib.logging.LearnLogRecord;

/**
 * Filters log messages based on categories. Non-LearnLib log records
 * are filed under category SYSTEM.
 * 
 * @author falkhowar
 */
public class CategoryFilter implements Filter {

    private final EnumSet<Category> categories;
    
    public CategoryFilter(EnumSet<Category> categories) {
        this.categories = categories;
    }
        
    @Override
    public boolean isLoggable(LogRecord record) {
        if (record.getClass() != LearnLogRecord.class) {
            return categories.contains(Category.SYSTEM);
        }
        LearnLogRecord lrec = (LearnLogRecord)record;        
        return categories.contains(lrec.getCategory());
   }
    
}
